//+------------------------------------------------------------------+
//|                                                 Projection bands |
//|                                                                  |
//|                                                                  |
//| original indicator described in TASC 13:7 article                |
//| "Signaling Change With Projection Bands" by Mel Widner Ph.D.     }
//|                                                                  |
//+------------------------------------------------------------------+

#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property  indicator_buffers 2
#property  indicator_color1  DeepSkyBlue
#property  indicator_color2  PaleVioletRed
#property  indicator_width2  2
//#property  indicator_style1  STYLE_DOT

//
//
//
//
//

extern string TimeFrame         = "Current time frame";
extern int    ProjectionPeriod  = 20;
extern double SignalPeriod      =  3;

//
//
//
//
//

double bufferPoc[];
double bufferSig[];
bool   returnBars;
int    timeFrame;
string indicatorFileName;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int init()
{
   SetIndexBuffer(0,bufferPoc);
   SetIndexBuffer(1,bufferSig);
         timeFrame  = stringToTimeFrame(TimeFrame);
         returnBars = (TimeFrame=="returnBars");
         indicatorFileName = WindowExpertName();
         IndicatorShortName(timeFrameToString(timeFrame)+" Projection oscilator ("+ProjectionPeriod+","+DoubleToStr(SignalPeriod,2)+")");
   return(0);
}
int deinit() { return(0); }


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
      if(counted_bars<0) return(-1);
      if(counted_bars>0) counted_bars--;
           int limit=MathMin(Bars-counted_bars,Bars-1);
           if (returnBars) { bufferPoc[0] = limit+1; return(0); }
           if (timeFrame!=Period())
           {
               limit = MathMin(Bars-1,MathMax(limit,iCustom(NULL,timeFrame,indicatorFileName,"returnBars",0,0)*timeFrame/Period()));
               for (int i=limit; i>=0; i--)
               {
                  int y = iBarShift(NULL,timeFrame,Time[i]);
                  bufferPoc[i] = iCustom(NULL,timeFrame,indicatorFileName,"",ProjectionPeriod,SignalPeriod,0,y);
                  bufferSig[i] = iCustom(NULL,timeFrame,indicatorFileName,"",ProjectionPeriod,SignalPeriod,1,y);
               }
               return(0);
           }

   //
   //
   //
   //
   //

   double alpha = 2.0/(1.0+SignalPeriod);
   for(i=limit; i>=0; i--)
   {
      double slopel = lrsSlope(ProjectionPeriod,PRICE_LOW ,i);
      double slopeh = lrsSlope(ProjectionPeriod,PRICE_HIGH,i);
         double max = iMA(NULL,0,1,0,MODE_SMA,PRICE_HIGH,i);
         double min = iMA(NULL,0,1,0,MODE_SMA,PRICE_LOW ,i);

         for (int k=1; k<ProjectionPeriod; k++)
         {
            min = MathMin(min,iMA(NULL,0,1,0,MODE_SMA,PRICE_LOW ,i+k)+k*slopel);
            max = MathMax(max,iMA(NULL,0,1,0,MODE_SMA,PRICE_HIGH,i+k)+k*slopeh);
         }
         if (max!=min)
               bufferPoc[i] = 100.0*(iMA(NULL,0,1,0,MODE_SMA,PRICE_CLOSE,i)-min)/(max-min);
         else  bufferPoc[i] = 0;
               bufferSig[i] = bufferSig[i+1]+alpha*(bufferPoc[i]-bufferSig[i+1]);
   }
   
   //
   //
   //
   //
   //
   
   return(0);
}


  
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//

double lrsSlope(int period,int appliedPrice,int shift)
{
   double sumx  = period*(period+1.0)/2.0;
   double sumx2 = period*(period+1.0)*(2*period+1.0)/6.0;
   double sumxy = 0.00;
   double sumy  = 0.00;

   int n = period;
   int i;
      for (i=0; i<period; i++,n--)
      {
         double price  = iMA(NULL,0,1,0,MODE_SMA,appliedPrice,shift+i);
                sumxy += n*price;
                sumy  +=   price;
      }        

   //
   //
   //
   //
   //
   
   double slope = 0.00;
   double top   = period*sumxy - sumx*sumy;
   double bot   = period*sumx2 - sumx*sumx;
      if (bot != 0) slope = top/bot;
   return(slope);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

//
//
//
//
//

int stringToTimeFrame(string tfs)
{
   tfs = stringUpperCase(tfs);
   for (int i=ArraySize(iTfTable)-1; i>=0; i--)
         if (tfs==sTfTable[i] || tfs==""+iTfTable[i]) return(MathMax(iTfTable[i],Period()));
                                                      return(Period());
}
string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}

//
//
//
//
//

string stringUpperCase(string str)
{
   string   s = str;

   for (int length=StringLen(str)-1; length>=0; length--)
   {
      int tchar = StringGetChar(s, length);
         if((tchar > 96 && tchar < 123) || (tchar > 223 && tchar < 256))
                     s = StringSetChar(s, length, tchar - 32);
         else if(tchar > -33 && tchar < 0)
                     s = StringSetChar(s, length, tchar + 224);
   }
   return(s);
}